@@ -1,6 +1,6 @@
 /* Xtensa GNU/Linux native support.
 
-   Copyright (C) 2007-2018 Free Software Foundation, Inc.
+   Copyright (C) 2007-2021 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -28,7 +28,7 @@
 #include <signal.h>
 #include <sys/user.h>
 #include <sys/ioctl.h>
-#include "gdb_wait.h"
+#include "gdbsupport/gdb_wait.h"
 #include <fcntl.h>
 #include <sys/procfs.h>
 #include "nat/gdb_ptrace.h"
@@ -45,6 +45,16 @@
    hardware-specific overlays.  */
 #include "xtensa-xtregs.c"
 
+class xtensa_linux_nat_target final : public linux_nat_target
+{
+public:
+  /* Add our register access methods.  */
+  void fetch_registers (struct regcache *, int) override;
+  void store_registers (struct regcache *, int) override;
+};
+
+static xtensa_linux_nat_target the_xtensa_linux_nat_target;
+
 void
 fill_gregset (const struct regcache *regcache,
 	      gdb_gregset_t *gregsetp, int regnum)
@@ -54,63 +64,54 @@ fill_gregset (const struct regcache *regcache,
   struct gdbarch *gdbarch = regcache->arch ();
 
   if (regnum == gdbarch_pc_regnum (gdbarch) || regnum == -1)
-    regcache_raw_collect (regcache, gdbarch_pc_regnum (gdbarch), &regs->pc);
+    regcache->raw_collect (gdbarch_pc_regnum (gdbarch), &regs->pc);
   if (regnum == gdbarch_ps_regnum (gdbarch) || regnum == -1)
-    regcache_raw_collect (regcache, gdbarch_ps_regnum (gdbarch), &regs->ps);
+    regcache->raw_collect (gdbarch_ps_regnum (gdbarch), &regs->ps);
 
   if (regnum == gdbarch_tdep (gdbarch)->wb_regnum || regnum == -1)
-    regcache_raw_collect (regcache,
-			  gdbarch_tdep (gdbarch)->wb_regnum,
-			  &regs->windowbase);
+    regcache->raw_collect (gdbarch_tdep (gdbarch)->wb_regnum,
+			   &regs->windowbase);
   if (regnum == gdbarch_tdep (gdbarch)->ws_regnum || regnum == -1)
-    regcache_raw_collect (regcache,
-			  gdbarch_tdep (gdbarch)->ws_regnum,
-			  &regs->windowstart);
+    regcache->raw_collect (gdbarch_tdep (gdbarch)->ws_regnum,
+			   &regs->windowstart);
   if (regnum == gdbarch_tdep (gdbarch)->lbeg_regnum || regnum == -1)
-    regcache_raw_collect (regcache,
-			  gdbarch_tdep (gdbarch)->lbeg_regnum,
-			  &regs->lbeg);
+    regcache->raw_collect (gdbarch_tdep (gdbarch)->lbeg_regnum,
+			   &regs->lbeg);
   if (regnum == gdbarch_tdep (gdbarch)->lend_regnum || regnum == -1)
-    regcache_raw_collect (regcache,
-			  gdbarch_tdep (gdbarch)->lend_regnum,
-			  &regs->lend);
+    regcache->raw_collect (gdbarch_tdep (gdbarch)->lend_regnum,
+			   &regs->lend);
   if (regnum == gdbarch_tdep (gdbarch)->lcount_regnum || regnum == -1)
-    regcache_raw_collect (regcache,
-			  gdbarch_tdep (gdbarch)->lcount_regnum,
-			  &regs->lcount);
+    regcache->raw_collect (gdbarch_tdep (gdbarch)->lcount_regnum,
+			   &regs->lcount);
   if (regnum == gdbarch_tdep (gdbarch)->sar_regnum || regnum == -1)
-    regcache_raw_collect (regcache,
-			  gdbarch_tdep (gdbarch)->sar_regnum,
-			  &regs->sar);
+    regcache->raw_collect (gdbarch_tdep (gdbarch)->sar_regnum,
+			   &regs->sar);
   if (regnum == gdbarch_tdep (gdbarch)->threadptr_regnum || regnum == -1)
-    regcache_raw_collect (regcache,
-			  gdbarch_tdep (gdbarch)->threadptr_regnum,
-			  &regs->threadptr);
+    regcache->raw_collect (gdbarch_tdep (gdbarch)->threadptr_regnum,
+			   &regs->threadptr);
   if (regnum >=gdbarch_tdep (gdbarch)->ar_base
       && regnum < gdbarch_tdep (gdbarch)->ar_base
 		    + gdbarch_tdep (gdbarch)->num_aregs)
-    regcache_raw_collect (regcache,regnum,
-			  &regs->ar[regnum - gdbarch_tdep (gdbarch)->ar_base]);
+    regcache->raw_collect (regnum,
+			   &regs->ar[regnum - gdbarch_tdep (gdbarch)->ar_base]);
   else if (regnum == -1)
     {
       for (i = 0; i < gdbarch_tdep (gdbarch)->num_aregs; ++i)
-	regcache_raw_collect (regcache,
-			      gdbarch_tdep (gdbarch)->ar_base + i,
-			      &regs->ar[i]);
+	regcache->raw_collect (gdbarch_tdep (gdbarch)->ar_base + i,
+			       &regs->ar[i]);
     }
   if (regnum >= gdbarch_tdep (gdbarch)->a0_base
       && regnum < gdbarch_tdep (gdbarch)->a0_base + C0_NREGS)
-    regcache_raw_collect (regcache, regnum,
-			  &regs->ar[(4 * regs->windowbase + regnum
-				     - gdbarch_tdep (gdbarch)->a0_base)
+    regcache->raw_collect (regnum,
+			   &regs->ar[(4 * regs->windowbase + regnum
+				      - gdbarch_tdep (gdbarch)->a0_base)
 			  % gdbarch_tdep (gdbarch)->num_aregs]);
   else if (regnum == -1)
     {
       for (i = 0; i < C0_NREGS; ++i)
-	regcache_raw_collect (regcache,
-			      gdbarch_tdep (gdbarch)->a0_base + i,
-			      &regs->ar[(4 * regs->windowbase + i)
-			      % gdbarch_tdep (gdbarch)->num_aregs]);
+	regcache->raw_collect (gdbarch_tdep (gdbarch)->a0_base + i,
+			       (&regs->ar[(4 * regs->windowbase + i)
+				% gdbarch_tdep (gdbarch)->num_aregs]));
     }
 }
 
@@ -124,63 +125,54 @@ supply_gregset_reg (struct regcache *regcache,
   struct gdbarch *gdbarch = regcache->arch ();
 
   if (regnum == gdbarch_pc_regnum (gdbarch) || regnum == -1)
-    regcache_raw_supply (regcache, gdbarch_pc_regnum (gdbarch), &regs->pc);
+    regcache->raw_supply (gdbarch_pc_regnum (gdbarch), &regs->pc);
   if (regnum == gdbarch_ps_regnum (gdbarch) || regnum == -1)
-    regcache_raw_supply (regcache, gdbarch_ps_regnum (gdbarch), &regs->ps);
+    regcache->raw_supply (gdbarch_ps_regnum (gdbarch), &regs->ps);
 
   if (regnum == gdbarch_tdep (gdbarch)->wb_regnum || regnum == -1)
-    regcache_raw_supply (regcache,
-			  gdbarch_tdep (gdbarch)->wb_regnum,
+    regcache->raw_supply (gdbarch_tdep (gdbarch)->wb_regnum,
 			  &regs->windowbase);
   if (regnum == gdbarch_tdep (gdbarch)->ws_regnum || regnum == -1)
-    regcache_raw_supply (regcache,
-			  gdbarch_tdep (gdbarch)->ws_regnum,
+    regcache->raw_supply (gdbarch_tdep (gdbarch)->ws_regnum,
 			  &regs->windowstart);
   if (regnum == gdbarch_tdep (gdbarch)->lbeg_regnum || regnum == -1)
-    regcache_raw_supply (regcache,
-			  gdbarch_tdep (gdbarch)->lbeg_regnum,
+    regcache->raw_supply (gdbarch_tdep (gdbarch)->lbeg_regnum,
 			  &regs->lbeg);
   if (regnum == gdbarch_tdep (gdbarch)->lend_regnum || regnum == -1)
-    regcache_raw_supply (regcache,
-			  gdbarch_tdep (gdbarch)->lend_regnum,
+    regcache->raw_supply (gdbarch_tdep (gdbarch)->lend_regnum,
 			  &regs->lend);
   if (regnum == gdbarch_tdep (gdbarch)->lcount_regnum || regnum == -1)
-    regcache_raw_supply (regcache,
-			  gdbarch_tdep (gdbarch)->lcount_regnum,
+    regcache->raw_supply (gdbarch_tdep (gdbarch)->lcount_regnum,
 			  &regs->lcount);
   if (regnum == gdbarch_tdep (gdbarch)->sar_regnum || regnum == -1)
-    regcache_raw_supply (regcache,
-			  gdbarch_tdep (gdbarch)->sar_regnum,
+    regcache->raw_supply (gdbarch_tdep (gdbarch)->sar_regnum,
 			  &regs->sar);
   if (regnum == gdbarch_tdep (gdbarch)->threadptr_regnum || regnum == -1)
-    regcache_raw_supply (regcache,
-			  gdbarch_tdep (gdbarch)->threadptr_regnum,
+    regcache->raw_supply (gdbarch_tdep (gdbarch)->threadptr_regnum,
 			  &regs->threadptr);
   if (regnum >=gdbarch_tdep (gdbarch)->ar_base
       && regnum < gdbarch_tdep (gdbarch)->ar_base
 		    + gdbarch_tdep (gdbarch)->num_aregs)
-    regcache_raw_supply (regcache,regnum,
+    regcache->raw_supply (regnum,
 			  &regs->ar[regnum - gdbarch_tdep (gdbarch)->ar_base]);
   else if (regnum == -1)
     {
       for (i = 0; i < gdbarch_tdep (gdbarch)->num_aregs; ++i)
-	regcache_raw_supply (regcache,
-			      gdbarch_tdep (gdbarch)->ar_base + i,
+	regcache->raw_supply (gdbarch_tdep (gdbarch)->ar_base + i,
 			      &regs->ar[i]);
     }
   if (regnum >= gdbarch_tdep (gdbarch)->a0_base
       && regnum < gdbarch_tdep (gdbarch)->a0_base + C0_NREGS)
-    regcache_raw_supply (regcache, regnum,
-			 &regs->ar[(4 * regs->windowbase + regnum
-				    - gdbarch_tdep (gdbarch)->a0_base)
+    regcache->raw_supply (regnum,
+			  &regs->ar[(4 * regs->windowbase + regnum
+				     - gdbarch_tdep (gdbarch)->a0_base)
 			 % gdbarch_tdep (gdbarch)->num_aregs]);
   else if (regnum == -1)
     {
       for (i = 0; i < C0_NREGS; ++i)
-	regcache_raw_supply (regcache,
-			     gdbarch_tdep (gdbarch)->a0_base + i,
-			     &regs->ar[(4 * regs->windowbase + i)
-			     % gdbarch_tdep (gdbarch)->num_aregs]);
+	regcache->raw_supply (gdbarch_tdep (gdbarch)->a0_base + i,
+			      &regs->ar[(4 * regs->windowbase + i)
+					% gdbarch_tdep (gdbarch)->num_aregs]);
     }
 }
 
@@ -210,7 +202,7 @@ supply_fpregset (struct regcache *regcache,
 static void
 fetch_gregs (struct regcache *regcache, int regnum)
 {
-  int tid = ptid_get_lwp (regcache_get_ptid (regcache));
+  int tid = regcache->ptid ().lwp ();
   gdb_gregset_t regs;
   int areg;
   
@@ -229,7 +221,7 @@ fetch_gregs (struct regcache *regcache, int regnum)
 static void
 store_gregs (struct regcache *regcache, int regnum)
 {
-  int tid = ptid_get_lwp (regcache_get_ptid (regcache));
+  int tid = regcache->ptid ().lwp ();
   gdb_gregset_t regs;
   int areg;
 
@@ -257,7 +249,7 @@ static int xtreg_high;
 static void
 fetch_xtregs (struct regcache *regcache, int regnum)
 {
-  int tid = ptid_get_lwp (regcache_get_ptid (regcache));
+  int tid = regcache->ptid ().lwp ();
   const xtensa_regtable_t *ptr;
   char xtregs [XTENSA_ELF_XTREG_SIZE];
 
@@ -266,14 +258,13 @@ fetch_xtregs (struct regcache *regcache, int regnum)
 
   for (ptr = xtensa_regmap_table; ptr->name; ptr++)
     if (regnum == ptr->gdb_regnum || regnum == -1)
-      regcache_raw_supply (regcache, ptr->gdb_regnum,
-			   xtregs + ptr->ptrace_offset);
+      regcache->raw_supply (ptr->gdb_regnum, xtregs + ptr->ptrace_offset);
 }
 
 static void
 store_xtregs (struct regcache *regcache, int regnum)
 {
-  int tid = ptid_get_lwp (regcache_get_ptid (regcache));
+  int tid = regcache->ptid ().lwp ();
   const xtensa_regtable_t *ptr;
   char xtregs [XTENSA_ELF_XTREG_SIZE];
 
@@ -282,16 +273,15 @@ store_xtregs (struct regcache *regcache, int regnum)
 
   for (ptr = xtensa_regmap_table; ptr->name; ptr++)
     if (regnum == ptr->gdb_regnum || regnum == -1)
-      regcache_raw_collect (regcache, ptr->gdb_regnum,
-			    xtregs + ptr->ptrace_offset);
+      regcache->raw_collect (ptr->gdb_regnum, xtregs + ptr->ptrace_offset);
 
   if (ptrace (PTRACE_SETXTREGS, tid, 0, (long)&xtregs) < 0)
     perror_with_name (_("Couldn't write extended registers"));
 }
 
-static void
-xtensa_linux_fetch_inferior_registers (struct target_ops *ops,
-				       struct regcache *regcache, int regnum)
+void
+xtensa_linux_nat_target::fetch_registers (struct regcache *regcache,
+					  int regnum)
 {
   if (regnum == -1)
     {
@@ -304,9 +294,9 @@ xtensa_linux_fetch_inferior_registers (struct target_ops *ops,
     fetch_xtregs (regcache, regnum);
 }
 
-static void
-xtensa_linux_store_inferior_registers (struct target_ops *ops,
-				       struct regcache *regcache, int regnum)
+void
+xtensa_linux_nat_target::store_registers (struct regcache *regcache,
+					  int regnum)
 {
   if (regnum == -1)
     {
@@ -323,7 +313,7 @@ xtensa_linux_store_inferior_registers (struct target_ops *ops,
 
 ps_err_e
 ps_get_thread_area (struct ps_prochandle *ph,
-                    lwpid_t lwpid, int idx, void **base)
+		    lwpid_t lwpid, int idx, void **base)
 {
   xtensa_elf_gregset_t regs;
 
@@ -338,10 +328,10 @@ ps_get_thread_area (struct ps_prochandle *ph,
   return PS_OK;
 }
 
+void _initialize_xtensa_linux_nat ();
 void
-_initialize_xtensa_linux_nat (void)
+_initialize_xtensa_linux_nat ()
 {
-  struct target_ops *t;
   const xtensa_regtable_t *ptr;
 
   /* Calculate the number range for extended registers.  */
@@ -355,12 +345,6 @@ _initialize_xtensa_linux_nat (void)
 	xtreg_high = ptr->gdb_regnum;
     }
 
-  /* Fill in the generic GNU/Linux methods.  */
-  t = linux_target ();
-
-  /* Add our register access methods.  */
-  t->to_fetch_registers = xtensa_linux_fetch_inferior_registers;
-  t->to_store_registers = xtensa_linux_store_inferior_registers;
-
-  linux_nat_add_target (t);
+  linux_target = &the_xtensa_linux_nat_target;
+  add_inf_child_target (&the_xtensa_linux_nat_target);
 }
